@@ -18,9 +18,9 @@
 
 #include "config.h"
 
+#include <adwaita.h>
 #include <gio/gdesktopappinfo.h>
 #include <glib/gi18n.h>
-#include <gtk/gtk.h>
 
 #include "screencastwidget.h"
 #include "displaystatetracker.h"
@@ -49,7 +49,9 @@ struct _ScreenCastWidget
 
   GtkWidget *window_heading;
   GtkWidget *window_list;
-  GtkWidget *window_list_scrolled;
+
+  GtkCheckButton *persist_check;
+  ScreenCastPersistMode persist_mode;
 
   DisplayStateTracker *display_state_tracker;
   gulong monitors_changed_handler_id;
@@ -58,6 +60,8 @@ struct _ScreenCastWidget
   gulong windows_changed_handler_id;
 
   guint selection_changed_timeout_id;
+  gboolean allow_multiple;
+  ScreenCastSourceType source_types;
 };
 
 static GQuark quark_monitor_widget_data;
@@ -65,18 +69,20 @@ static GQuark quark_window_widget_data;
 
 G_DEFINE_TYPE (ScreenCastWidget, screen_cast_widget, GTK_TYPE_BOX)
 
+/*
+ * Auxiliary methods
+ */
+
 static GtkWidget *
 create_window_widget (Window *window)
 {
-  GtkWidget *window_widget;
-  GtkWidget *window_label;
   GtkWidget *window_image;
+  GtkWidget *check_image;
+  GtkWidget *row;
   GIcon *icon = NULL;
   g_autoptr(GDesktopAppInfo) info = NULL;
+  g_autofree char *escaped_name = NULL;
 
-  window_widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
-  gtk_widget_set_margin_start (window_widget, 12);
-  gtk_widget_set_margin_end (window_widget, 12);
   info = g_desktop_app_info_new (window_get_app_id (window));
   if (info != NULL)
     icon = g_app_info_get_icon (G_APP_INFO (info));
@@ -84,54 +90,80 @@ create_window_widget (Window *window)
     icon = g_themed_icon_new ("application-x-executable");
   window_image = gtk_image_new_from_gicon (icon);
   gtk_image_set_pixel_size (GTK_IMAGE (window_image), 32);
-  gtk_widget_set_margin_start (window_image, 12);
-  gtk_widget_set_margin_end (window_image, 12);
 
-  gtk_box_append (GTK_BOX (window_widget), window_image);
+  check_image = gtk_image_new_from_icon_name ("object-select-symbolic");
+  gtk_widget_hide (check_image);
 
-  window_label = gtk_label_new (window_get_title (window));
-  gtk_widget_set_margin_top (window_label, 12);
-  gtk_widget_set_margin_bottom (window_label, 12);
-  gtk_box_append (GTK_BOX (window_widget), window_label);
+  row = adw_action_row_new ();
+  adw_action_row_add_prefix (ADW_ACTION_ROW (row), window_image);
+  adw_action_row_add_suffix (ADW_ACTION_ROW (row), check_image);
 
-  g_object_set_qdata (G_OBJECT (window_widget),
+  escaped_name = g_markup_escape_text (window_get_title (window), -1);
+  adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), escaped_name);
+
+  g_object_set_qdata (G_OBJECT (row),
                       quark_window_widget_data,
                       window);
-  return window_widget;
+  g_object_set_data (G_OBJECT (row), "check", check_image);
+  return row;
 }
 
 static GtkWidget *
 create_monitor_widget (LogicalMonitor *logical_monitor)
 {
-  GtkWidget *monitor_widget;
+  g_autoptr(GString) string = NULL;
+  GtkWidget *check_image;
+  GtkWidget *row;
   GList *l;
 
-  monitor_widget = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
-  gtk_widget_set_margin_start (monitor_widget, 12);
-  gtk_widget_set_margin_end (monitor_widget, 12);
+  check_image = gtk_image_new_from_icon_name ("object-select-symbolic");
+  gtk_widget_hide (check_image);
 
+  row = adw_action_row_new ();
+
+  string = g_string_new (NULL);
   for (l = logical_monitor_get_monitors (logical_monitor); l; l = l->next)
     {
       Monitor *monitor = l->data;
-      GtkWidget *monitor_label;
 
       if (!l->prev)
-        g_object_set_qdata (G_OBJECT (monitor_widget),
+        g_object_set_qdata (G_OBJECT (row),
                             quark_monitor_widget_data,
                             monitor);
 
-      monitor_label = gtk_label_new (monitor_get_display_name (monitor));
-      gtk_widget_set_margin_top (monitor_label, 12);
-      gtk_widget_set_margin_bottom (monitor_label, 12);
-      gtk_box_append (GTK_BOX (monitor_widget), monitor_label);
+      g_string_append (string, monitor_get_display_name (monitor));
+
+      if (l->next)
+        g_string_append (string, "\n");
     }
 
-  gtk_widget_show (monitor_widget);
-  return monitor_widget;
+  adw_action_row_add_suffix (ADW_ACTION_ROW (row), check_image);
+  adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), string->str);
+  g_object_set_data (G_OBJECT (row), "check", check_image);
+
+  return row;
+}
+
+
+static GtkWidget *
+create_virtual_widget (void)
+{
+  GtkWidget *check_image;
+  GtkWidget *row;
+
+  check_image = gtk_image_new_from_icon_name ("object-select-symbolic");
+  gtk_widget_hide (check_image);
+
+  row = adw_action_row_new ();
+  adw_action_row_add_suffix (ADW_ACTION_ROW (row), check_image);
+  adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), _("Virtual monitor"));
+  g_object_set_data (G_OBJECT (row), "check", check_image);
+
+  return row;
 }
 
 static gboolean
-should_skip_window (Window *window,
+should_skip_window (Window    *window,
                     GtkWindow *toplevel)
 {
   g_autofree char *processed_app_id = NULL;
@@ -160,14 +192,8 @@ update_windows_list (ScreenCastWidget *widget)
   GList *windows;
   GList *l;
 
-  child = gtk_widget_get_first_child (GTK_WIDGET (window_list));
-  while (child)
-    {
-      GtkWidget *next = gtk_widget_get_next_sibling (child);
-
-      gtk_list_box_remove (window_list, child);
-      child = next;
-    }
+  while ((child = gtk_widget_get_first_child (GTK_WIDGET (window_list))) != NULL)
+    gtk_list_box_remove (window_list, child);
 
   toplevel = gtk_widget_get_ancestor (GTK_WIDGET (widget), GTK_TYPE_WINDOW);
   if (!toplevel)
@@ -192,44 +218,44 @@ update_monitors_list (ScreenCastWidget *widget)
 {
   GtkListBox *monitor_list = GTK_LIST_BOX (widget->monitor_list);
   GtkWidget *child;
-  GList *logical_monitors;
-  GList *l;
 
-  child = gtk_widget_get_first_child (GTK_WIDGET (monitor_list));
-  while (child)
+  while ((child = gtk_widget_get_first_child (GTK_WIDGET (monitor_list))) != NULL)
+    gtk_list_box_remove (monitor_list, child);
+
+  if (widget->source_types & SCREEN_CAST_SOURCE_TYPE_MONITOR)
     {
-      GtkWidget *next = gtk_widget_get_next_sibling (child);
+      GList *logical_monitors;
+      GList *l;
 
-      gtk_list_box_remove (monitor_list, child);
-      child = next;
+      logical_monitors =
+        display_state_tracker_get_logical_monitors (widget->display_state_tracker);
+      for (l = logical_monitors; l; l = l->next)
+        {
+          LogicalMonitor *logical_monitor = l->data;
+          GtkWidget *monitor_widget;
+
+          monitor_widget = create_monitor_widget (logical_monitor);
+          gtk_list_box_append (monitor_list, monitor_widget);
+        }
     }
 
-  logical_monitors =
-    display_state_tracker_get_logical_monitors (widget->display_state_tracker);
-  for (l = logical_monitors; l; l = l->next)
-    {
-      LogicalMonitor *logical_monitor = l->data;
-      GtkWidget *monitor_widget;
-
-      monitor_widget = create_monitor_widget (logical_monitor);
-      gtk_list_box_append (monitor_list, monitor_widget);
-    }
+  if (widget->source_types & SCREEN_CAST_SOURCE_TYPE_VIRTUAL)
+    gtk_list_box_append (monitor_list, create_virtual_widget ());
 }
 
 static gboolean
 is_row_selected (GtkListBoxRow *row)
 {
-  return GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row),
-                                             "is-row-selected"));
+  GtkWidget *check_image = g_object_get_data (G_OBJECT (row), "check");
+  return gtk_widget_get_visible (check_image);
 }
 
 static void
 set_row_is_selected (GtkListBoxRow *row,
                      gboolean is_selected)
 {
-  g_object_set_data (G_OBJECT (row),
-                     "is-row-selected",
-                     GINT_TO_POINTER (is_selected));
+  GtkWidget *check_image = g_object_get_data (G_OBJECT (row), "check");
+  gtk_widget_set_visible (check_image, is_selected);
 }
 
 static void
@@ -254,7 +280,7 @@ unselect_rows (GtkListBox *listbox)
 }
 
 static void
-on_windows_changed (ShellIntrospect *shell_introspect,
+on_windows_changed (ShellIntrospect  *shell_introspect,
                     ScreenCastWidget *widget)
 {
   update_windows_list (widget);
@@ -283,9 +309,9 @@ disconnect_windows_changed_listener (ScreenCastWidget *widget)
 }
 
 static void
-on_stack_switch (GtkStack *stack,
+on_stack_switch (GtkStack   *stack,
                  GParamSpec *pspec,
-                 gpointer *data)
+                 gpointer   *data)
 {
   ScreenCastWidget *widget = (ScreenCastWidget *)data;
   GtkWidget *visible_child;
@@ -307,9 +333,9 @@ on_stack_switch (GtkStack *stack,
 }
 
 static void
-on_row_activated (GtkListBox *box,
+on_row_activated (GtkListBox    *box,
                   GtkListBoxRow *row,
-                  gpointer *data)
+                  gpointer      *data)
 {
   if (!row)
     return;
@@ -378,7 +404,7 @@ schedule_selection_change (ScreenCastWidget *widget)
 }
 
 static void
-on_selected_rows_changed (GtkListBox *box,
+on_selected_rows_changed (GtkListBox       *box,
                           ScreenCastWidget *widget)
 {
   /* GtkListBox activates rows after selecting them, which prevents
@@ -386,95 +412,117 @@ on_selected_rows_changed (GtkListBox *box,
   schedule_selection_change (widget);
 }
 
-static void
-update_list_box_header (GtkListBoxRow *row,
-                        GtkListBoxRow *before,
-                        gpointer user_data)
-{
-  GtkWidget *header;
-
-  if (before)
-    header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-  else
-    header = NULL;
-
-  gtk_list_box_row_set_header (row, header);
-}
-
 static void
 on_monitors_changed (DisplayStateTracker *display_state_tracker,
-                     ScreenCastWidget *widget)
+                     ScreenCastWidget    *widget)
 {
   update_monitors_list (widget);
 }
 
-static gboolean
-add_selections (ScreenCastWidget *widget,
-                GVariantBuilder *source_selections_builder)
+/*
+ * GObject overrides
+ */
+
+static void
+screen_cast_widget_finalize (GObject *object)
 {
-  GList *selected_monitor_rows;
-  GList *selected_window_rows;
-  GList *l;
+  ScreenCastWidget *widget = SCREEN_CAST_WIDGET (object);
 
-  selected_monitor_rows =
-    gtk_list_box_get_selected_rows (GTK_LIST_BOX (widget->monitor_list));
-  selected_window_rows =
-    gtk_list_box_get_selected_rows (GTK_LIST_BOX (widget->window_list));
-  if (!selected_monitor_rows && !selected_window_rows)
-    return FALSE;
+  g_signal_handler_disconnect (widget->display_state_tracker,
+                               widget->monitors_changed_handler_id);
 
-  for (l = selected_monitor_rows; l; l = l->next)
+  if (widget->windows_changed_handler_id)
+    disconnect_windows_changed_listener (widget);
+
+  if (widget->selection_changed_timeout_id > 0)
     {
-      GtkWidget *monitor_widget = gtk_list_box_row_get_child (l->data);
-      Monitor *monitor;
-
-      monitor = g_object_get_qdata (G_OBJECT (monitor_widget),
-                                    quark_monitor_widget_data);
-
-      g_variant_builder_add (source_selections_builder, "(us)",
-                             SCREEN_CAST_SOURCE_TYPE_MONITOR,
-                             monitor_get_connector (monitor));
+      g_source_remove (widget->selection_changed_timeout_id);
+      widget->selection_changed_timeout_id = 0;
     }
-  g_list_free (selected_monitor_rows);
-  for (l = selected_window_rows; l; l = l->next)
-    {
-      GtkWidget *window_widget = gtk_list_box_row_get_child (l->data);
-      Window *window;
 
-      window = g_object_get_qdata (G_OBJECT (window_widget),
-                                    quark_window_widget_data);
-
-      g_variant_builder_add (source_selections_builder, "(ut)",
-                             SCREEN_CAST_SOURCE_TYPE_WINDOW,
-                             window_get_id (window));
-    }
-  g_list_free (selected_window_rows);
-
-  return TRUE;
+  G_OBJECT_CLASS (screen_cast_widget_parent_class)->finalize (object);
 }
 
-void
-screen_cast_widget_add_selections (ScreenCastWidget *widget,
-                                   GVariantBuilder *selections_builder)
+static void
+screen_cast_widget_class_init (ScreenCastWidgetClass *klass)
 {
-  GVariantBuilder source_selections_builder;
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-  g_variant_builder_init (&source_selections_builder, G_VARIANT_TYPE ("a(u?)"));
-  if (!add_selections (widget, &source_selections_builder))
-    {
-      g_variant_builder_clear (&source_selections_builder);
-    }
-  else
-    {
-      g_variant_builder_add (selections_builder, "{sv}",
-                             "selected_screen_cast_sources",
-                             g_variant_builder_end (&source_selections_builder));
-    }
+  object_class->finalize = screen_cast_widget_finalize;
+
+  signals[HAS_SELECTION_CHANGED] = g_signal_new ("has-selection-changed",
+                                                 G_TYPE_FROM_CLASS (klass),
+                                                 G_SIGNAL_RUN_LAST,
+                                                 0,
+                                                 NULL, NULL,
+                                                 NULL,
+                                                 G_TYPE_NONE, 1,
+                                                 G_TYPE_BOOLEAN);
+
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/freedesktop/portal/desktop/gnome/screencastwidget.ui");
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, persist_check);
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, source_type_switcher);
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, source_type);
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, monitor_selection);
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, window_selection);
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, monitor_heading);
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, monitor_list);
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, window_heading);
+  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, window_list);
+
+  quark_monitor_widget_data = g_quark_from_static_string ("-monitor-widget-connector-quark");
+  quark_window_widget_data = g_quark_from_static_string ("-window-widget-connector-quark");
+}
+
+static void
+screen_cast_widget_init (ScreenCastWidget *widget)
+{
+  gtk_widget_init_template (GTK_WIDGET (widget));
+
+  screen_cast_widget_set_app_id (widget, NULL);
+  screen_cast_widget_set_allow_multiple (widget, FALSE);
+
+  g_signal_connect (widget->source_type, "notify::visible-child",
+                    G_CALLBACK (on_stack_switch),
+                    widget);
+  g_signal_connect (widget->monitor_list, "row-activated",
+                    G_CALLBACK (on_row_activated),
+                    NULL);
+  g_signal_connect (widget->window_list, "row-activated",
+                    G_CALLBACK (on_row_activated),
+                    NULL);
+  g_signal_connect (widget->monitor_list, "selected-rows-changed",
+                    G_CALLBACK (on_selected_rows_changed),
+                    widget);
+  g_signal_connect (widget->window_list, "selected-rows-changed",
+                    G_CALLBACK (on_selected_rows_changed),
+                    widget);
+
+  widget->display_state_tracker = display_state_tracker_get ();
+  widget->monitors_changed_handler_id =
+    g_signal_connect (widget->display_state_tracker,
+                      "monitors-changed",
+                      G_CALLBACK (on_monitors_changed),
+                      widget);
+  widget->shell_introspect = shell_introspect_get ();
+
+  update_monitors_list (widget);
+  update_windows_list (widget);
+
+  gtk_widget_show (widget->monitor_list);
+  gtk_widget_show (widget->window_list);
+}
+
+GtkWidget *
+screen_cast_widget_get_stack_switcher (ScreenCastWidget *widget)
+{
+  return widget->source_type_switcher;
 }
 
 void
 screen_cast_widget_set_app_id (ScreenCastWidget *widget,
-                               const char *app_id)
+                               const char       *app_id)
 {
   g_autofree char *monitor_heading = NULL;
   g_autofree char *window_heading = NULL;
@@ -508,8 +556,10 @@ screen_cast_widget_set_app_id (ScreenCastWidget *widget,
 
 void
 screen_cast_widget_set_allow_multiple (ScreenCastWidget *widget,
-                                       gboolean multiple)
+                                       gboolean          multiple)
 {
+  widget->allow_multiple = multiple;
+
   gtk_list_box_set_selection_mode (GTK_LIST_BOX (widget->monitor_list),
                                    multiple ? GTK_SELECTION_MULTIPLE
                                             : GTK_SELECTION_SINGLE);
@@ -519,125 +569,101 @@ screen_cast_widget_set_allow_multiple (ScreenCastWidget *widget,
 }
 
 void
-screen_cast_widget_set_source_types (ScreenCastWidget *screen_cast_widget,
-                                     ScreenCastSourceType source_types)
+screen_cast_widget_set_source_types (ScreenCastWidget     *screen_cast_widget,
+                                     ScreenCastSourceType  source_types)
 {
+  screen_cast_widget->source_types = source_types;
+
   if (source_types & SCREEN_CAST_SOURCE_TYPE_MONITOR)
     gtk_widget_show (screen_cast_widget->monitor_selection);
 
   if (source_types & SCREEN_CAST_SOURCE_TYPE_WINDOW)
     gtk_widget_show (screen_cast_widget->window_selection);
 
+  if (source_types & SCREEN_CAST_SOURCE_TYPE_VIRTUAL)
+    gtk_widget_show (screen_cast_widget->monitor_selection);
+
   if (__builtin_popcount (source_types) > 1)
     gtk_widget_show (screen_cast_widget->source_type_switcher);
+
+  update_monitors_list (screen_cast_widget);
 }
 
-static void
-screen_cast_widget_finalize (GObject *object)
+GPtrArray *
+screen_cast_widget_get_selected_streams (ScreenCastWidget *self)
 {
-  ScreenCastWidget *widget = SCREEN_CAST_WIDGET (object);
+  ScreenCastStreamInfo *info;
+  g_autoptr(GPtrArray) streams = NULL;
+  g_autoptr(GList) selected_monitor_rows = NULL;
+  g_autoptr(GList) selected_window_rows = NULL;
+  uint32_t id = 0;
+  GList *l;
 
-  g_signal_handler_disconnect (widget->display_state_tracker,
-                               widget->monitors_changed_handler_id);
+  streams = g_ptr_array_new_with_free_func (g_free);
 
-  if (widget->windows_changed_handler_id)
-    disconnect_windows_changed_listener (widget);
+  selected_monitor_rows =
+    gtk_list_box_get_selected_rows (GTK_LIST_BOX (self->monitor_list));
+  selected_window_rows =
+    gtk_list_box_get_selected_rows (GTK_LIST_BOX (self->window_list));
 
-  if (widget->selection_changed_timeout_id > 0)
+  if (!selected_monitor_rows && !selected_window_rows)
+    return g_steal_pointer (&streams);
+
+  for (l = selected_monitor_rows; l; l = l->next)
     {
-      g_source_remove (widget->selection_changed_timeout_id);
-      widget->selection_changed_timeout_id = 0;
+
+      Monitor *monitor;
+
+      monitor = g_object_get_qdata (G_OBJECT (l->data), quark_monitor_widget_data);
+
+      if (monitor)
+        {
+          info = g_new0 (ScreenCastStreamInfo, 1);
+          info->type = SCREEN_CAST_SOURCE_TYPE_MONITOR;
+          info->data.monitor = monitor;
+          info->id = id++;
+          g_ptr_array_add (streams, info);
+        }
+      else
+        {
+          info = g_new0 (ScreenCastStreamInfo, 1);
+          info->type = SCREEN_CAST_SOURCE_TYPE_VIRTUAL;
+          info->id = id++;
+          g_ptr_array_add (streams, info);
+        }
     }
 
-  G_OBJECT_CLASS (screen_cast_widget_parent_class)->finalize (object);
-}
+  for (l = selected_window_rows; l; l = l->next)
+    {
+      Window *window;
 
-static void
-screen_cast_widget_init (ScreenCastWidget *widget)
-{
-  GtkScrolledWindow *scrolled_window;
-  GtkAdjustment *vadjustment;
+      window = g_object_get_qdata (G_OBJECT (l->data), quark_window_widget_data);
 
-  gtk_widget_init_template (GTK_WIDGET (widget));
+      info = g_new0 (ScreenCastStreamInfo, 1);
+      info->type = SCREEN_CAST_SOURCE_TYPE_WINDOW;
+      info->data.window = window;
+      info->id = id++;
+      g_ptr_array_add (streams, info);
+    }
 
-  screen_cast_widget_set_app_id (widget, NULL);
-  screen_cast_widget_set_allow_multiple (widget, FALSE);
-
-  gtk_list_box_set_header_func (GTK_LIST_BOX (widget->monitor_list),
-                                update_list_box_header,
-                                NULL, NULL);
-  gtk_list_box_set_header_func (GTK_LIST_BOX (widget->window_list),
-                                update_list_box_header,
-                                NULL, NULL);
-  scrolled_window = GTK_SCROLLED_WINDOW (widget->window_list_scrolled);
-  vadjustment = gtk_scrolled_window_get_vadjustment (scrolled_window);
-  gtk_list_box_set_adjustment (GTK_LIST_BOX (widget->window_list), vadjustment);
-
-  g_signal_connect (widget->source_type, "notify::visible-child",
-                    G_CALLBACK (on_stack_switch),
-                    widget);
-  g_signal_connect (widget->monitor_list, "row-activated",
-                    G_CALLBACK (on_row_activated),
-                    NULL);
-  g_signal_connect (widget->window_list, "row-activated",
-                    G_CALLBACK (on_row_activated),
-                    NULL);
-  g_signal_connect (widget->monitor_list, "selected-rows-changed",
-                    G_CALLBACK (on_selected_rows_changed),
-                    widget);
-  g_signal_connect (widget->window_list, "selected-rows-changed",
-                    G_CALLBACK (on_selected_rows_changed),
-                    widget);
-
-  widget->display_state_tracker = display_state_tracker_get ();
-  widget->monitors_changed_handler_id =
-    g_signal_connect (widget->display_state_tracker,
-                      "monitors-changed",
-                      G_CALLBACK (on_monitors_changed),
-                      widget);
-  widget->shell_introspect = shell_introspect_get ();
-
-  update_monitors_list (widget);
-  update_windows_list (widget);
-
-  gtk_widget_show (widget->monitor_list);
-  gtk_widget_show (widget->window_list);
-}
-
-static void
-screen_cast_widget_class_init (ScreenCastWidgetClass *klass)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
-
-  object_class->finalize = screen_cast_widget_finalize;
-
-  signals[HAS_SELECTION_CHANGED] = g_signal_new ("has-selection-changed",
-                                                 G_TYPE_FROM_CLASS (klass),
-                                                 G_SIGNAL_RUN_LAST,
-                                                 0,
-                                                 NULL, NULL,
-                                                 NULL,
-                                                 G_TYPE_NONE, 1,
-                                                 G_TYPE_BOOLEAN);
-
-  gtk_widget_class_set_template_from_resource (widget_class, "/org/freedesktop/portal/desktop/gnome/screencastwidget.ui");
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, source_type_switcher);
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, source_type);
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, monitor_selection);
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, window_selection);
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, monitor_heading);
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, monitor_list);
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, window_heading);
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, window_list);
-  gtk_widget_class_bind_template_child (widget_class, ScreenCastWidget, window_list_scrolled);
-
-  quark_monitor_widget_data = g_quark_from_static_string ("-monitor-widget-connector-quark");
-  quark_window_widget_data = g_quark_from_static_string ("-window-widget-connector-quark");
+  return g_steal_pointer (&streams);
 }
 
 void
-init_screen_cast_widget (void)
+screen_cast_widget_set_persist_mode (ScreenCastWidget      *screen_cast_widget,
+                                     ScreenCastPersistMode  persist_mode)
 {
-  g_type_ensure (screen_cast_widget_get_type ());
+  screen_cast_widget->persist_mode = persist_mode;
+
+  gtk_widget_set_visible (GTK_WIDGET (screen_cast_widget->persist_check),
+                                      persist_mode != SCREEN_CAST_PERSIST_MODE_NONE);
+}
+
+ScreenCastPersistMode
+screen_cast_widget_get_persist_mode (ScreenCastWidget *screen_cast_widget)
+{
+  if (!gtk_check_button_get_active (screen_cast_widget->persist_check))
+    return SCREEN_CAST_PERSIST_MODE_NONE;
+
+  return screen_cast_widget->persist_mode;
 }
